import {
  formatFiles,
  generateFiles,
  joinPathFragments,
  OverwriteStrategy,
  readProjectConfiguration,
  Tree,
} from '@nx/devkit';
import execa from 'execa';
import os from 'node:os';
import fs from 'node:fs/promises';
import path from 'node:path';

import { TomgGeneratorSchema } from './schema';

export async function tomgGenerator(tree: Tree, options: TomgGeneratorSchema) {
  const libraryRoot = readProjectConfiguration(tree, options.project).root;

  const tmpDirPath = await fs.mkdtemp(path.join(os.tmpdir(), 'tomg-'));
  const { stdout, stderr } = await execa(
    `typeorm-model-generator`,
    [
      '-o',
      tmpDirPath,
      '-e',
      options.driver,
      '-h',
      options.host,
      '-d',
      options.database,
      '-u',
      options.username,
      '-x',
      options.password,
    ],
    { cwd: libraryRoot }
  );
  if (!options.quiet) {
    console.log(stdout);
    console.error(stderr);
  }

  generateFiles(
    tree,
    tmpDirPath,
    joinPathFragments(libraryRoot, options.directory),
    options,
    { overwriteStrategy: OverwriteStrategy.Overwrite }
  );

  await formatFiles(tree);
  await fs.rm(tmpDirPath, { recursive: true, force: true });

  console.log();
  console.log('Generation complete.');
  console.log();
}

export default tomgGenerator;
